@@ -253,7 +253,7 @@ gpm_stats_update_smooth_data (GPtrArray *list)
 	convolved = egg_array_float_convolve (outliers, gaussian);
 
 	/* add the smoothed data back into a new array */
-	new = g_ptr_array_new ();
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
 	for (i=0; i<list->len; i++) {
 		point = (GpmPointObj *) g_ptr_array_index (list, i);
 		point_new = g_new0 (GpmPointObj, 1);
@@ -303,6 +303,31 @@ gpm_stats_bool_to_text (gboolean ret)
 	return ret ? _("Yes") : _("No");
 }
 
+/**
+ * gpm_stats_get_printable_device_path:
+ **/
+static gchar *
+gpm_stats_get_printable_device_path (DkpDevice *device)
+{
+	const gchar *prefix = "/org/freedesktop/DeviceKit/Power/devices/";
+	const gchar *object_path;
+	gchar *device_path = NULL;
+	guint len;
+
+	/* get object path */
+	object_path = dkp_device_get_object_path (device);
+	if (object_path == NULL)
+		goto out;
+	if (!g_str_has_prefix (object_path, prefix))
+		goto out;
+
+	/* trim */
+	len = strlen (prefix);
+	device_path = g_strdup (object_path+len);
+out:
+	return device_path;
+}
+
 /**
  * gpm_stats_update_info_page_details:
  **/
@@ -335,6 +360,7 @@ gpm_stats_update_info_page_details (DkpDevice *device)
 	gchar *vendor = NULL;
 	gchar *serial = NULL;
 	gchar *model = NULL;
+	gchar *device_path = NULL;
 
 	gtk_list_store_clear (list_store_info);
 
@@ -368,7 +394,11 @@ gpm_stats_update_info_page_details (DkpDevice *device)
 	time_tm = localtime (&t);
 	strftime (time_buf, sizeof time_buf, "%c", time_tm);
 
-	gpm_stats_add_info_data (_("Device"), dkp_device_get_object_path (device));
+	/* remove prefix */
+	device_path = gpm_stats_get_printable_device_path (device);
+	gpm_stats_add_info_data (_("Device"), device_path);
+	g_free (device_path);
+
 	gpm_stats_add_info_data (_("Type"), gpm_device_type_to_localised_text (type, 1));
 	if (vendor != NULL && vendor[0] != '\0')
 		gpm_stats_add_info_data (_("Vendor"), vendor);
@@ -480,8 +510,7 @@ gpm_stats_set_graph_data (GtkWidget *widget, GPtrArray *data, gboolean use_smoot
 		if (use_points)
 			gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_POINTS, data);
 		gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_LINE, smoothed);
-		g_ptr_array_foreach (smoothed, (GFunc) gpm_point_obj_free, NULL);
-		g_ptr_array_free (smoothed, TRUE);
+		g_ptr_array_unref (smoothed);
 	}
 
 	/* show */
@@ -505,8 +534,8 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 	gint32 offset = 0;
 	GTimeVal timeval;
 
-	new = g_ptr_array_new ();
-	if (strcmp (history_type, GPM_HISTORY_CHARGE_VALUE) == 0) {
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
+	if (g_strcmp0 (history_type, GPM_HISTORY_CHARGE_VALUE) == 0) {
 		g_object_set (graph_history,
 			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
 			      "type-y", GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
@@ -515,7 +544,7 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 			      "stop-x", 0,
 			      "autorange-y", TRUE,
 			      NULL);
-	} else if (strcmp (history_type, GPM_HISTORY_RATE_VALUE) == 0) {
+	} else if (g_strcmp0 (history_type, GPM_HISTORY_RATE_VALUE) == 0) {
 		g_object_set (graph_history,
 			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
 			      "type-y", GPM_GRAPH_WIDGET_TYPE_POWER,
@@ -535,11 +564,17 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 			      NULL);
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_history_nodata"));
 	array = dkp_device_get_history (device, history_type, history_time, 150, NULL);
 	if (array == NULL) {
+		/* show no data label and hide graph */
 		gtk_widget_hide (graph_history);
+		gtk_widget_show (widget);
 		goto out;
 	}
+
+	/* hide no data and show graph */
+	gtk_widget_hide (widget);
 	gtk_widget_show (graph_history);
 
 	g_get_current_time (&timeval);
@@ -559,8 +594,14 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 			point->color = egg_color_from_rgb (255, 0, 0);
 		else if (hobj->state == DKP_DEVICE_STATE_DISCHARGING)
 			point->color = egg_color_from_rgb (0, 0, 255);
+#if DKP_CHECK_VERSION(0x009)
+		else if (hobj->state == DKP_DEVICE_STATE_PENDING_CHARGE)
+			point->color = egg_color_from_rgb (200, 0, 0);
+		else if (hobj->state == DKP_DEVICE_STATE_PENDING_DISCHARGE)
+			point->color = egg_color_from_rgb (0, 0, 200);
+#endif
 		else {
-			if (strcmp (history_type, GPM_HISTORY_RATE_VALUE) == 0)
+			if (g_strcmp0 (history_type, GPM_HISTORY_RATE_VALUE) == 0)
 				point->color = egg_color_from_rgb (255, 255, 255);
 			else
 				point->color = egg_color_from_rgb (0, 255, 0);
@@ -578,10 +619,8 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 	/* present data to graph */
 	gpm_stats_set_graph_data (graph_history, new, checked, points);
 
-	g_ptr_array_foreach (array, (GFunc) dkp_history_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
-	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
-	g_ptr_array_free (new, TRUE);
+	g_ptr_array_unref (array);
+	g_ptr_array_unref (new);
 out:
 	return;
 }
@@ -603,17 +642,17 @@ gpm_stats_update_info_page_stats (DkpDevice *device)
 	gboolean use_data = FALSE;
 	const gchar *type = NULL;
 
-	new = g_ptr_array_new ();
-	if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0) {
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
+	if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0) {
 		type = "charging";
 		use_data = TRUE;
-	} else if (strcmp (stats_type, GPM_STATS_DISCHARGE_DATA_VALUE) == 0) {
+	} else if (g_strcmp0 (stats_type, GPM_STATS_DISCHARGE_DATA_VALUE) == 0) {
 		type = "discharging";
 		use_data = TRUE;
-	} else if (strcmp (stats_type, GPM_STATS_CHARGE_ACCURACY_VALUE) == 0) {
+	} else if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_ACCURACY_VALUE) == 0) {
 		type = "charging";
 		use_data = FALSE;
-	} else if (strcmp (stats_type, GPM_STATS_DISCHARGE_ACCURACY_VALUE) == 0) {
+	} else if (g_strcmp0 (stats_type, GPM_STATS_DISCHARGE_ACCURACY_VALUE) == 0) {
 		type = "discharging";
 		use_data = FALSE;
 	} else {
@@ -636,12 +675,19 @@ gpm_stats_update_info_page_stats (DkpDevice *device)
 			      NULL);
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_stats_nodata"));
 	array = dkp_device_get_statistics (device, type, NULL);
 	if (array == NULL) {
+		/* show no data label and hide graph */
 		gtk_widget_hide (graph_statistics);
+		gtk_widget_show (widget);
 		goto out;
 	}
 
+	/* hide no data and show graph */
+	gtk_widget_hide (widget);
+	gtk_widget_show (graph_statistics);
+
 	for (i=0; i<array->len; i++) {
 		sobj = (DkpStatsObj *) g_ptr_array_index (array, i);
 		point = gpm_point_obj_new ();
@@ -664,10 +710,8 @@ gpm_stats_update_info_page_stats (DkpDevice *device)
 	/* present data to graph */
 	gpm_stats_set_graph_data (graph_statistics, new, checked, points);
 
-	g_ptr_array_foreach (array, (GFunc) dkp_stats_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
-	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
-	g_ptr_array_free (new, TRUE);
+	g_ptr_array_unref (array);
+	g_ptr_array_unref (new);
 out:
 	return;
 }
@@ -754,27 +798,27 @@ gpm_stats_format_cmdline (const DkpWakeupsObj *obj)
 	}
 
 	/* common kernel cmd names */
-	if (strcmp (obj->cmdline, "insmod") == 0) {
+	if (g_strcmp0 (obj->cmdline, "insmod") == 0) {
 		/* TRANSLATORS: kernel module, usually a device driver */
 		temp_ptr = _("Kernel module");
 		goto out;
 	}
-	if (strcmp (obj->cmdline, "modprobe") == 0) {
+	if (g_strcmp0 (obj->cmdline, "modprobe") == 0) {
 		/* TRANSLATORS: kernel module, usually a device driver */
 		temp_ptr = _("Kernel module");
 		goto out;
 	}
-	if (strcmp (obj->cmdline, "swapper") == 0) {
+	if (g_strcmp0 (obj->cmdline, "swapper") == 0) {
 		/* TRANSLATORS: kernel housekeeping */
 		temp_ptr = _("Kernel core");
 		goto out;
 	}
-	if (strcmp (obj->cmdline, "kernel-ipi") == 0) {
+	if (g_strcmp0 (obj->cmdline, "kernel-ipi") == 0) {
 		/* TRANSLATORS: interrupt between processors */
 		temp_ptr = _("Interprocessor interrupt");
 		goto out;
 	}
-	if (strcmp (obj->cmdline, "interrupt") == 0) {
+	if (g_strcmp0 (obj->cmdline, "interrupt") == 0) {
 		/* TRANSLATORS: unknown interrupt */
 		temp_ptr = _("Interrupt");
 		goto out;
@@ -817,19 +861,19 @@ gpm_stats_format_details (const DkpWakeupsObj *obj)
 	gchar *details;
 
 	/* replace common driver names */
-	if (strcmp (obj->details, "i8042") == 0) {
+	if (g_strcmp0 (obj->details, "i8042") == 0) {
 		/* TRANSLATORS: the keyboard and mouse device event */
 		details = g_strdup (_("PS/2 keyboard/mouse/touchpad"));
-	} else if (strcmp (obj->details, "acpi") == 0) {
+	} else if (g_strcmp0 (obj->details, "acpi") == 0) {
 		/* TRANSLATORS: ACPI, the Intel power standard on laptops and desktops */
 		details = g_strdup (_("ACPI"));
-	} else if (strcmp (obj->details, "ata_piix") == 0) {
+	} else if (g_strcmp0 (obj->details, "ata_piix") == 0) {
 		/* TRANSLATORS: serial ATA is a new style of hard disk interface */
 		details = g_strdup (_("Serial ATA"));
-	} else if (strcmp (obj->details, "libata") == 0) {
+	} else if (g_strcmp0 (obj->details, "libata") == 0) {
 		/* TRANSLATORS: this is the old-style ATA interface */
 		details = g_strdup (_("ATA host controller"));
-	} else if (strcmp (obj->details, "iwl3945") == 0 || strcmp (obj->details, "iwlagn") == 0) {
+	} else if (g_strcmp0 (obj->details, "iwl3945") == 0 || g_strcmp0 (obj->details, "iwlagn") == 0) {
 		/* TRANSLATORS: 802.11 wireless adaptor */
 		details = g_strdup (_("Intel wireless adaptor"));
 
@@ -978,8 +1022,7 @@ gpm_stats_update_wakeups_data (void)
 		obj = g_ptr_array_index (array, i);
 		gpm_stats_add_wakeups_obj (obj);
 	}
-	g_ptr_array_foreach (array, (GFunc) dkp_wakeups_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
+	g_ptr_array_unref (array);
 }
 
 static void
@@ -1153,7 +1196,7 @@ gpm_stats_device_changed_cb (DkpClient *client, DkpDevice *device, gpointer user
 	if (object_path == NULL || current_device == NULL)
 		return;
 	egg_debug ("changed:   %s", object_path);
-	if (strcmp (current_device, object_path) == 0)
+	if (g_strcmp0 (current_device, object_path) == 0)
 		gpm_stats_update_info_data (device);
 }
 
@@ -1170,7 +1213,7 @@ gpm_stats_device_removed_cb (DkpClient *client, DkpDevice *device, gpointer user
 
 	object_path = dkp_device_get_object_path (device);
 	egg_debug ("removed:   %s", object_path);
-	if (strcmp (current_device, object_path) == 0) {
+	if (g_strcmp0 (current_device, object_path) == 0) {
 		gtk_list_store_clear (list_store_info);
 	}
 
@@ -1178,7 +1221,7 @@ gpm_stats_device_removed_cb (DkpClient *client, DkpDevice *device, gpointer user
 	ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list_store_devices), &iter);
 	do {
 		gtk_tree_model_get (GTK_TREE_MODEL (list_store_devices), &iter, GPM_DEVICES_COLUMN_ID, &id, -1);
-		if (strcmp (id, object_path) == 0) {
+		if (g_strcmp0 (id, object_path) == 0) {
 			gtk_list_store_remove (list_store_devices, &iter);
 			break;
 		}
@@ -1197,25 +1240,25 @@ gpm_stats_history_type_combo_changed_cb (GtkWidget *widget, gpointer data)
 	const gchar *axis_x = NULL;
 	const gchar *axis_y = NULL;
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
-	if (strcmp (value, GPM_HISTORY_RATE_TEXT) == 0) {
+	if (g_strcmp0 (value, GPM_HISTORY_RATE_TEXT) == 0) {
 		history_type = GPM_HISTORY_RATE_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Time elapsed");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Power");
-	} else if (strcmp (value, GPM_HISTORY_CHARGE_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_HISTORY_CHARGE_TEXT) == 0) {
 		history_type = GPM_HISTORY_CHARGE_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Time elapsed");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Cell charge");
-	} else if (strcmp (value, GPM_HISTORY_TIME_FULL_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_HISTORY_TIME_FULL_TEXT) == 0) {
 		history_type = GPM_HISTORY_TIME_FULL_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Time elapsed");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Predicted time");
-	} else if (strcmp (value, GPM_HISTORY_TIME_EMPTY_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_HISTORY_TIME_EMPTY_TEXT) == 0) {
 		history_type = GPM_HISTORY_TIME_EMPTY_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Time elapsed");
@@ -1248,25 +1291,25 @@ gpm_stats_type_combo_changed_cb (GtkWidget *widget, gpointer data)
 	const gchar *axis_x = NULL;
 	const gchar *axis_y = NULL;
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
-	if (strcmp (value, GPM_STATS_CHARGE_DATA_TEXT) == 0) {
+	if (g_strcmp0 (value, GPM_STATS_CHARGE_DATA_TEXT) == 0) {
 		stats_type = GPM_STATS_CHARGE_DATA_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Cell charge");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Correction factor");
-	} else if (strcmp (value, GPM_STATS_CHARGE_ACCURACY_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_STATS_CHARGE_ACCURACY_TEXT) == 0) {
 		stats_type = GPM_STATS_CHARGE_ACCURACY_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Cell charge");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Prediction accuracy");
-	} else if (strcmp (value, GPM_STATS_DISCHARGE_DATA_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_STATS_DISCHARGE_DATA_TEXT) == 0) {
 		stats_type = GPM_STATS_DISCHARGE_DATA_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Cell charge");
 		/* TRANSLATORS: this is the Y axis on the graph */
 		axis_y = _("Correction factor");
-	} else if (strcmp (value, GPM_STATS_DISCHARGE_ACCURACY_TEXT) == 0) {
+	} else if (g_strcmp0 (value, GPM_STATS_DISCHARGE_ACCURACY_TEXT) == 0) {
 		stats_type = GPM_STATS_DISCHARGE_ACCURACY_VALUE;
 		/* TRANSLATORS: this is the X axis on the graph */
 		axis_x = _("Cell charge");
@@ -1297,13 +1340,13 @@ gpm_stats_range_combo_changed (GtkWidget *widget, gpointer data)
 {
 	gchar *value;
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
-	if (strcmp (value, GPM_HISTORY_MINUTE_TEXT) == 0)
+	if (g_strcmp0 (value, GPM_HISTORY_MINUTE_TEXT) == 0)
 		history_time = GPM_HISTORY_MINUTE_VALUE;
-	else if (strcmp (value, GPM_HISTORY_HOUR_TEXT) == 0)
+	else if (g_strcmp0 (value, GPM_HISTORY_HOUR_TEXT) == 0)
 		history_time = GPM_HISTORY_HOUR_VALUE;
-	else if (strcmp (value, GPM_HISTORY_DAY_TEXT) == 0)
+	else if (g_strcmp0 (value, GPM_HISTORY_DAY_TEXT) == 0)
 		history_time = GPM_HISTORY_DAY_VALUE;
-	else if (strcmp (value, GPM_HISTORY_WEEK_TEXT) == 0)
+	else if (g_strcmp0 (value, GPM_HISTORY_WEEK_TEXT) == 0)
 		history_time = GPM_HISTORY_WEEK_VALUE;
 	else
 		g_assert (FALSE);
@@ -1403,6 +1446,7 @@ main (int argc, char *argv[])
 	DkpClient *client;
 	GPtrArray *devices;
 	DkpDevice *device;
+	DkpDeviceType type;
 	guint i;
 	gint page;
 	const gchar *object_path;
@@ -1441,7 +1485,7 @@ main (int argc, char *argv[])
 
 	/* are we already activated? */
 	egg_unique = egg_unique_new ();
-	ret = egg_unique_assign (egg_unique, "org.freedesktop.DeviceKit.Gnome");
+	ret = egg_unique_assign (egg_unique, "org.gnome.PowerManager.Statistics");
 	if (!ret)
 		goto unique_out;
 	g_signal_connect (egg_unique, "activated",
@@ -1470,14 +1514,14 @@ main (int argc, char *argv[])
 	box = GTK_BOX (gtk_builder_get_object (builder, "hbox_history"));
 	graph_history = gpm_graph_widget_new ();
 	gtk_box_pack_start (box, graph_history, TRUE, TRUE, 0);
-	gtk_widget_set_size_request (graph_history, 600, 300);
+	gtk_widget_set_size_request (graph_history, 400, 250);
 	gtk_widget_show (graph_history);
 
 	/* add statistics graph */
 	box = GTK_BOX (gtk_builder_get_object (builder, "hbox_statistics"));
 	graph_statistics = gpm_graph_widget_new ();
 	gtk_box_pack_start (box, graph_statistics, TRUE, TRUE, 0);
-	gtk_widget_set_size_request (graph_statistics, 600, 300);
+	gtk_widget_set_size_request (graph_statistics, 400, 250);
 	gtk_widget_show (graph_statistics);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_stats"));
@@ -1580,7 +1624,7 @@ main (int argc, char *argv[])
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_CHARGE_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_TIME_FULL_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_TIME_EMPTY_TEXT);
-	if (strcmp (history_type, GPM_HISTORY_RATE_VALUE) == 0)
+	if (g_strcmp0 (history_type, GPM_HISTORY_RATE_VALUE) == 0)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
 	else
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
@@ -1593,13 +1637,13 @@ main (int argc, char *argv[])
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_CHARGE_ACCURACY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_DISCHARGE_DATA_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_DISCHARGE_ACCURACY_TEXT);
-	if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
+	if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
-	else if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
+	else if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
-	else if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
+	else if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 2);
-	else if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
+	else if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
 	else
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 3);
@@ -1634,24 +1678,56 @@ main (int argc, char *argv[])
 	devices = dkp_client_enumerate_devices (client, NULL);
 	if (devices == NULL)
 		goto out;
+
+	/* add devices in visually pleasing order */
 	for (i=0; i < devices->len; i++) {
 		device = g_ptr_array_index (devices, i);
-		gpm_stats_add_device (device);
-		if (i == 0) {
-			gpm_stats_update_info_data (device);
-			current_device = g_strdup (dkp_device_get_object_path (device));
-		}
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_LINE_POWER)
+			gpm_stats_add_device (device);
+	}
+	for (i=0; i < devices->len; i++) {
+		device = g_ptr_array_index (devices, i);
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_BATTERY)
+			gpm_stats_add_device (device);
+	}
+	for (i=0; i < devices->len; i++) {
+		device = g_ptr_array_index (devices, i);
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_UPS)
+			gpm_stats_add_device (device);
+	}
+	for (i=0; i < devices->len; i++) {
+		device = g_ptr_array_index (devices, i);
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_MONITOR)
+			gpm_stats_add_device (device);
+	}
+	for (i=0; i < devices->len; i++) {
+		device = g_ptr_array_index (devices, i);
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_MOUSE)
+			gpm_stats_add_device (device);
+	}
+	for (i=0; i < devices->len; i++) {
+		device = g_ptr_array_index (devices, i);
+		g_object_get (device, "type", &type, NULL);
+		if (type == DKP_DEVICE_TYPE_KEYBOARD)
+			gpm_stats_add_device (device);
+	}
+
+	/* set current device */
+	if (devices->len > 0) {
+		device = g_ptr_array_index (devices, 0);
+		gpm_stats_update_info_data (device);
+		current_device = g_strdup (dkp_device_get_object_path (device));
 	}
 
 	last_device = gconf_client_get_string (gconf_client, GPM_CONF_INFO_LAST_DEVICE, NULL);
 
-#if 0
-	/* FIXME: this is only available in DeviceKit 008 */
+	/* has capability to measure wakeups */
 	ret = dkp_wakeups_has_capability (wakeups);
-#else
-	/* can we get wakeup data? */
-	ret = TRUE;
-#endif
 	if (ret) {
 		GtkTreeIter iter;
 		gtk_list_store_append (list_store_devices, &iter);
@@ -1667,7 +1743,7 @@ main (int argc, char *argv[])
 		object_path = (const gchar *) g_ptr_array_index (devices, i);
 		if (last_device == NULL || object_path == NULL)
 			break;
-		if (strcmp (last_device, object_path) == 0) {
+		if (g_strcmp0 (last_device, object_path) == 0) {
 			GtkTreePath *path;
 			gchar *path_str;
 			path_str = g_strdup_printf ("%i", i);
@@ -1678,9 +1754,7 @@ main (int argc, char *argv[])
 			gtk_tree_path_free (path);
 		}
 	}
-
-	g_ptr_array_foreach (devices, (GFunc) g_object_unref, NULL);
-	g_ptr_array_free (devices, TRUE);
+	g_ptr_array_unref (devices);
 
 	/* set axis */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_history_type"));
